// file      : tests/cxx/parser/list/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test xsd:list parsing.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct string_list_parser: string_list<void, string>
{
  virtual void
  pre ()
  {
    cout << "{" << endl;
  }

  virtual void
  item (string const& v)
  {
    cout << "  '" << v << "'" << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct type_parser: type<void, void>
{
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;
    string_list_parser string_list_p;
    type_parser type_p;

    string_list_p.parsers (string_p);
    type_p.parsers (string_list_p);

    xml_schema::document<void> doc_p (type_p, "test", "root");

    // Use dont_validate flag to disable whitespace collapsing.
    //
    doc_p.parse (argv[1], xml_schema::flags::dont_validate);
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
