// file      : tests/cxx/parser/name-clash/inheritance/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test for name clashes across inheritance hierarchy.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct derived_parser: derived<void, string, string>
{
  virtual void
  pre ()
  {
  }

  virtual void
  e (string const& v)
  {
    cout << "e: " << v << endl;
  }

  virtual void
  e1 (string const& v)
  {
    cout << "e1: " << v << endl;
  }

  virtual void
  post ()
  {
  }
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;

    derived_parser derived_p;

    derived_p.parsers (string_p, string_p);

    xml_schema::document<void> doc_p (derived_p, "test", "root");

    doc_p.parse (argv[1]);
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
