// file      : tests/cxx/parser/validation/attribute/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test attribute and attribute wildcard (anyAttribute) validation.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct pass_a_parser: pass_a<void, string, string>
{
  virtual void
  pre ()
  {
    cout << "pass-a" << endl
         << "{" << endl;
  }

  virtual void
  a (string const& v)
  {
    cout << "  a = " << v << endl;
  }

  virtual void
  b (string const& v)
  {
    cout << "  b = " << v << endl;
  }

  virtual void
  _any_attribute (xml_schema::ro_string const& ns,
                  xml_schema::ro_string const& name,
                  xml_schema::ro_string const& value)
  {
    cout << "  any: " << ns << "#" << name << " = " << value << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct pass_b_parser: pass_b<void, string, string>
{
  virtual void
  pre ()
  {
    cout << "pass-b" << endl
         << "{" << endl;
  }

  virtual void
  a (string const& v)
  {
    cout << "  a = " << v << endl;
  }

  virtual void
  b (string const& v)
  {
    cout << "  b = " << v << endl;
  }

  virtual void
  _any_attribute (xml_schema::ro_string const& ns,
                  xml_schema::ro_string const& name,
                  xml_schema::ro_string const& value)
  {
    cout << "  any: " << ns << "#" << name << " = " << value << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct pass_c_parser: pass_c<void, string>
{
  virtual void
  pre ()
  {
    cout << "pass-c" << endl
         << "{" << endl;
  }

  virtual void
  a (string const& v)
  {
    cout << "  a = " << v << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct fail_parser: fail<void, string>
{
  virtual void
  pre ()
  {
    cout << "fail" << endl
         << "{" << endl;
  }

  virtual void
  a (string const& v)
  {
    cout << "  a = " << v << endl;
  }

  virtual void
  post ()
  {
    cout << "}" << endl
         << endl;
  }
};

struct type_parser: type<void,
                         void, // pass-a
                         void, // pass-b
                         void, // pass-c
                         void> // fail
{
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::string string_p;
    pass_a_parser pass_a_p;
    pass_b_parser pass_b_p;
    pass_c_parser pass_c_p;
    fail_parser fail_p;
    type_parser type_p;

    pass_a_p.parsers (string_p, string_p);
    pass_b_p.parsers (string_p, string_p);
    pass_c_p.parsers (string_p);
    fail_p.parsers (string_p);
    type_p.parsers (pass_a_p, pass_b_p, pass_c_p, fail_p);

    xml_schema::document<void> doc_p (type_p, "test", "root");

    try
    {
      doc_p.parse (argv[1], xml_schema::flags::dont_validate);
    }
    catch (xml_schema::expected_attribute const& e)
    {
      if (e.expected_name () == "a" && e.expected_namespace ().empty ())
      {
        cout << "  missing attribute detected" << endl
             << "}" << endl;
      }
      else
        throw;
    }
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
