// file      : tests/cxx/parser/validation/built-in/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

// Test built-in type validation and parsing.
//

#include <string>
#include <iostream>

#include "test.hxx"

using namespace std;
using namespace test;

struct type_parser: type<void,
                         signed char,
                         unsigned char,
                         short,
                         unsigned short,
                         int,
                         unsigned int,
                         long long,
                         unsigned long long,
                         bool,
                         float,
                         double,
                         long double,
                         string>
{
  virtual void
  byte (signed char const& v)
  {
    cout << short (v) << endl;
  }

  virtual void
  unsigned_byte (unsigned char const& v)
  {
    cout << (unsigned short) (v) << endl;
  }

  virtual void
  short_ (short const& v)
  {
    cout << v << endl;
  }

  virtual void
  unsigned_short (unsigned short const& v)
  {
    cout << v << endl;
  }

  virtual void
  int_ (int const& v)
  {
    cout << v << endl;
  }

  virtual void
  unsigned_int (unsigned int const& v)
  {
    cout << v << endl;
  }

  virtual void
  long_ (long long const& v)
  {
    cout << v << endl;
  }

  virtual void
  unsigned_long (unsigned long long const& v)
  {
    cout << v << endl;
  }

  virtual void
  boolean (bool const& v)
  {
    cout << v << endl;
  }

  virtual void
  float_ (float const& v)
  {
    cout << v << endl;
  }

  virtual void
  double_ (double const& v)
  {
    cout << v << endl;
  }

  virtual void
  decimal_ (long double const& v)
  {
    cout << v << endl;
  }

  virtual void
  string (std::string const& v)
  {
    cout << "'" << v << "'" << endl;
  }
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " test.xml" << endl;
    return 1;
  }

  try
  {
    xml_schema::byte byte_p;
    xml_schema::unsigned_byte unsigned_byte_p;
    xml_schema::short_ short_p;
    xml_schema::unsigned_short unsigned_short_p;
    xml_schema::int_ int_p;
    xml_schema::unsigned_int unsigned_int_p;
    xml_schema::long_ long_p;
    xml_schema::unsigned_long unsigned_long_p;
    xml_schema::boolean boolean_p;
    xml_schema::float_ float_p;
    xml_schema::double_ double_p;
    xml_schema::decimal decimal_p;
    xml_schema::string string_p;

    type_parser type_p;

    type_p.parsers (byte_p,
                    unsigned_byte_p,
                    short_p,
                    unsigned_short_p,
                    int_p,
                    unsigned_int_p,
                    long_p,
                    unsigned_long_p,
                    boolean_p,
                    float_p,
                    double_p,
                    decimal_p,
                    string_p);

    xml_schema::document<void> doc_p (type_p, "test", "root");

    doc_p.parse (argv[1], xml_schema::flags::dont_validate);
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
