// file      : xsd/cxx/tree/parser-source.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <cxx/tree/parser-source.hxx>

#include <xsd-frontend/semantic-graph.hxx>
#include <xsd-frontend/traversal.hxx>

namespace CXX
{
  namespace Tree
  {
    namespace
    {
      struct Element: Traversal::Element,
                      GlobalElementBase,
                      protected virtual Context
      {
        Element (Context& c)
            : GlobalElementBase (c), Context (c)
        {
        }

        virtual Void
        traverse (Type& e)
        {
          if (!doc_root_p (e))
            return;

          String const& name (ename (e));
          String properties_type (L"::xsd::cxx::tree::properties< " +
                                  char_type + L" >");

          // Note that I am using fq-name in function calls because g++ gets
          // confused if the name is 'type'. (see tests/schema/anonymous)
          //

          // URI.
          //
          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const " << string_type << "& u," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::auto_initializer i (" << endl
             << "(f & " << flags_type << "::dont_initialize) == 0," << endl
             << "(f & " << flags_type << "::keep_dom) == 0);"
             << endl
             << "::xsd::cxx::tree::error_handler< " << char_type << " > h;"
             << endl
             << "::xsd::cxx::xml::dom::auto_ptr< " <<
            "::xercesc::DOMDocument > d (" << endl
             << "::xsd::cxx::xml::dom::parse< " << char_type << " > (" <<
            "u, h, p, f));"
             << endl
             << "h.throw_if_failed< ::xsd::cxx::tree::parsing< " <<
            char_type << " > > ();"
             << endl
             << "return " << fq_name (e) << " (" << endl
             << "static_cast< const ::xercesc::DOMDocument& > (*d), f);"
             << "}";


          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const " << string_type << "& u," << endl
             << "::xsd::cxx::xml::error_handler< " <<
            char_type << " >& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::auto_initializer i (" << endl
             << "(f & " << flags_type << "::dont_initialize) == 0," << endl
             << "(f & " << flags_type << "::keep_dom) == 0);"
             << endl
             << "::xsd::cxx::xml::dom::auto_ptr< " <<
            "::xercesc::DOMDocument > d (" << endl
             << "::xsd::cxx::xml::dom::parse< " << char_type << " > (" <<
            "u, h, p, f));"
             << endl
             << "if (!d)"
             << "{"
             << "throw ::xsd::cxx::tree::parsing< " <<
            char_type << " > ();"
             << "}"
             << "return " << fq_name (e) << " (" << endl
             << "static_cast< const ::xercesc::DOMDocument& > (*d), f);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const " << string_type << "& u," << endl
             << "::xercesc::DOMErrorHandler& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::dom::auto_ptr< " <<
            "::xercesc::DOMDocument > d (" << endl
             << "::xsd::cxx::xml::dom::parse< " << char_type << " > (" <<
            "u, h, p, f));"
             << endl
             << "if (!d)"
             << "{"
             << "throw ::xsd::cxx::tree::parsing< " <<
            char_type << " > ();"
             << "}"
             << "return " << fq_name (e) << " (" << endl
             << "static_cast< const ::xercesc::DOMDocument& > (*d), f);"
             << "}";


          // istream
          //
          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (::std::istream& is," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::auto_initializer i (" << endl
             << "(f & " << flags_type << "::dont_initialize) == 0," << endl
             << "(f & " << flags_type << "::keep_dom) == 0);"
             << endl
             << "::xsd::cxx::xml::sax::std_input_source isrc (is);"
             << "::xercesc::Wrapper4InputSource wrap (&isrc, false);"
             << "return " << fq_name (e) << " (wrap, f, p);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (::std::istream& is," << endl
             << "::xsd::cxx::xml::error_handler< " <<
            char_type << " >& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::auto_initializer i (" << endl
             << "(f & " << flags_type << "::dont_initialize) == 0," << endl
             << "(f & " << flags_type << "::keep_dom) == 0);"
             << endl
             << "::xsd::cxx::xml::sax::std_input_source isrc (is);"
             << "::xercesc::Wrapper4InputSource wrap (&isrc, false);"
             << "return " << fq_name (e) << " (wrap, h, f, p);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (::std::istream& is," << endl
             << "::xercesc::DOMErrorHandler& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::sax::std_input_source isrc (is);"
             << "::xercesc::Wrapper4InputSource wrap (&isrc, false);"
             << "return " << fq_name (e) << " (wrap, h, f, p);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (::std::istream& is," << endl
             << "const " << string_type << "& sid," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::auto_initializer i (" << endl
             << "(f & " << flags_type << "::dont_initialize) == 0," << endl
             << "(f & " << flags_type << "::keep_dom) == 0);"
             << endl
             << "::xsd::cxx::xml::sax::std_input_source isrc (is, sid);"
             << "::xercesc::Wrapper4InputSource wrap (&isrc, false);"
             << "return " << fq_name (e) << " (wrap, f, p);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (::std::istream& is," << endl
             << "const " << string_type << "& sid," << endl
             << "::xsd::cxx::xml::error_handler< " <<
            char_type << " >& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::auto_initializer i (" << endl
             << "(f & " << flags_type << "::dont_initialize) == 0," << endl
             << "(f & " << flags_type << "::keep_dom) == 0);"
             << endl
             << "::xsd::cxx::xml::sax::std_input_source isrc (is, sid);"
             << "::xercesc::Wrapper4InputSource wrap (&isrc, false);"
             << "return " << fq_name (e) << " (wrap, h, f, p);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (::std::istream& is," << endl
             << "const " << string_type << "& sid," << endl
             << "::xercesc::DOMErrorHandler& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::sax::std_input_source isrc (is, sid);"
             << "::xercesc::Wrapper4InputSource wrap (&isrc, false);"
             << "return " << fq_name (e) << " (wrap, h, f, p);"
             << "}";


          // InputSource.
          //
          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const ::xercesc::DOMInputSource& i," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::tree::error_handler< " << char_type << " > h;"
             << endl
             << "::xsd::cxx::xml::dom::auto_ptr< " <<
            "::xercesc::DOMDocument > d (" << endl
             << "::xsd::cxx::xml::dom::parse< " << char_type << " > (" <<
            "i, h, p, f));"
             << endl
             << "h.throw_if_failed< ::xsd::cxx::tree::parsing< " <<
            char_type << " > > ();"
             << endl
             << "return " << fq_name (e) << " (" << endl
             << "static_cast< const ::xercesc::DOMDocument& > (*d), f, p);"
             << "}";

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const ::xercesc::DOMInputSource& i," << endl
             << "::xsd::cxx::xml::error_handler< " <<
            char_type << " >& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::dom::auto_ptr< " <<
            "::xercesc::DOMDocument > d (" << endl
             << "::xsd::cxx::xml::dom::parse< " << char_type << " > (" <<
            "i, h, p, f));"
             << endl
             << "if (!d)"
             << "{"
             << "throw ::xsd::cxx::tree::parsing< " <<
            char_type << " > ();"
             << "}"
             << "return " << fq_name (e) << " (" << endl
             << "static_cast< const ::xercesc::DOMDocument& > (*d), f, p);"
             << "}";


          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const ::xercesc::DOMInputSource& i," << endl
             << "::xercesc::DOMErrorHandler& h," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "& p)"
             << "{"
             << "::xsd::cxx::xml::dom::auto_ptr< " <<
            "::xercesc::DOMDocument > d (" << endl
             << "::xsd::cxx::xml::dom::parse< " << char_type << " > (" <<
            "i, h, p, f));"
             << endl
             << "if (!d)"
             << "{"
             << "throw ::xsd::cxx::tree::parsing< " <<
            char_type << " > ();"
             << "}"
             << "return " << fq_name (e) << " (" << endl
             << "static_cast< const ::xercesc::DOMDocument& > (*d), f, p);"
             << "}";


          // DOM.
          //

          Boolean fund (false);

          {
            IsFundamentalType t (fund);
            t.dispatch (e.type ());
          }

          // Check if we need to handle xsi:type and substitution groups.
          // If this element's type is anonymous or mapped to a fundamental
          // C++ type then we don't need to do anything.
          //
          Boolean polymorphic (
            options.value<CLI::generate_polymorphic> () &&
            e.type ().named () &&
            !fund);

          os << "::std::auto_ptr< " << type_name (e) << " >" << endl
             << name << " (const ::xercesc::DOMDocument& d," << endl
             << flags_type << " f," << endl
             << "const " << properties_type << "&)"
             << "{"
             << "::xsd::cxx::xml::dom::auto_ptr< ::xercesc::DOMDocument > " <<
            "c (" << endl
             << "(f & " << flags_type << "::keep_dom) " << endl
             << "? static_cast< ::xercesc::DOMDocument* > (" <<
            "d.cloneNode (true))" << endl
             << ": 0);"
             << endl
             << "const " << xml_element_type << " e (" << endl
             << "c.get ()" << endl
             << "? *c->getDocumentElement ()" << endl
             << ": *d.getDocumentElement ());"
             << endl;

          if (polymorphic)
          {
            // aCC cannot handle an inline call to type_factory_map_instance.
            //
            os << "::xsd::cxx::tree::type_factory_map< " << char_type <<
              " >& tfm (" << endl
               << "::xsd::cxx::tree::type_factory_map_instance< 0, " <<
              char_type << " > ());"
               << endl
               << "::std::auto_ptr< " << type_name (e) << " > r (" << endl
               << "tfm.create< " << type_name (e) << " > (" << endl
               << L << "\"" << e.name () << "\"," << endl
               << L << "\"" << e.namespace_().name () << "\"," << endl
               << "true," << endl
               << "true," << endl
               << "e," << endl
               << "f," << endl
               << "0));"
               << endl
               << "if (r.get () != 0)"
               << "{";
          }
          else
          {
            os << "if (e.name () == " << L << "\"" << e.name () << "\"" <<
              " &&" << endl
               << "e.namespace_ () == " << L << "\"" << e.namespace_().name () << "\")"
               << "{";

            if (fund)
            {
              os << "::std::auto_ptr< " << type_name (e) << " > r (" << endl
                 << "new " << type_name (e) << " (" << endl
                 << "::xsd::cxx::tree::traits< " << type_name (e) << ", " <<
                char_type << " >::create (" << endl
                 << "e.dom_element (), f, 0)));";
            }
            else
            {
              os << "::std::auto_ptr< " << type_name (e) << " > r (" << endl
                 << "::xsd::cxx::tree::traits< " << type_name (e) << ", " <<
                char_type << " >::create (" << endl
                 << "e.dom_element (), f, 0));";
            }
          }

          os << "if (f & " << flags_type << "::keep_dom) c.release ();"
             << "return r;"
             << "}"
             << "throw ::xsd::cxx::tree::unexpected_element < " <<
            char_type << " > (" << endl
             << "e.name ()," << endl
             << "e.namespace_ ()," << endl
             << L << "\"" << e.name () << "\"," << endl
             << L << "\"" << e.namespace_().name () << "\");"
             << "}";
        }

      private:
        String
        type_name (Type& e)
        {
          SemanticGraph::Type& t (e.type ());

          if (t.named ())
          {
            std::wostringstream o;

            MemberTypeName type (*this, o);
            type.dispatch (t);

            return o.str ();
          }
          else
          {
            String const& name (ename (e));
            return name + L"::" + unclash (name, "type");
          }
        }
      };
    }

    Void
    generate_parser_source (Context& ctx,
                            UnsignedLong first,
                            UnsignedLong last)
    {
      ctx.os << "#include <istream>" << endl
             << "#include <xercesc/framework/Wrapper4InputSource.hpp>" << endl
             << "#include <xsd/cxx/xml/sax/std-input-source.hxx>" << endl
             << "#include <xsd/cxx/tree/error-handler.hxx>" << endl
             << endl;

      Traversal::Schema schema;
      Traversal::Sources sources;
      Traversal::Names names_ns, names;
      Namespace ns (ctx, false, first, last);
      Element element (ctx);

      schema >> sources >> schema;
      schema >> names_ns >> ns >> names >> element;

      schema.dispatch (ctx.schema_root);
    }
  }
}
